@@ -23,6 +23,12 @@ rowCumsums.sirt <- function(matr){
 # The C code was posted by Romain Francois at
 # http://lists.r-forge.r-project.org/pipermail/rcpp-devel/2010-October/001198.html
 
+##########################################################################
+# rowwise cumulative sum
+colCumsums.sirt <- function(matr){ 
+	t( rowCumsums.sirt( t(matr) ) )
+					}
+
 ##########################################################################
 #****					
 # 'interval_index' searches an index when a frequency is exceeded
@@ -50,4 +56,23 @@ rowKSmallest.sirt <- function( matr , K , break.ties=TRUE){
     return(a1)
         }
 ##########################################################################
-
+# Ksmallest -> different implementation
+rowKSmallest2.sirt <- function(matr , K ){
+    Nmis <- nrow(matr)
+    disty <- matr
+    donors <- K
+    indvec <- 1:Nmis
+    M1 <- max(disty)+1
+    smallval <- donor.ind <- matrix( 0 , nrow=Nmis , ncol=donors )
+    res1 <- rowMins.sirt(matr=disty)
+    donor.ind[,1] <- res1$minind
+    smallval[,1] <- res1$minval    
+    for (ii in 2:donors){
+        disty[ cbind(indvec , donor.ind[,ii-1] ) ] <- M1
+        res1 <- rowMins.sirt(matr=disty)
+        donor.ind[,ii] <- res1$minind
+        smallval[,ii] <- res1$minval    
+                        }          
+    res <- list( "smallval"=smallval , "smallind" = donor.ind )
+    return(res)
+    }
